# Copyright 2024 Klira AI
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""LangGraph adapter.

Inherits from LangChainAdapter since LangGraph is built on LangChain.
"""

from __future__ import annotations

import logging
from typing import Any, Callable

from klira.sdk.adapters.langchain import LangChainAdapter

logger = logging.getLogger(__name__)


class LangGraphAdapter(LangChainAdapter):
    """Adapter for LangGraph (extends LangChain adapter)."""

    @property
    def framework_name(self) -> str:
        return "langgraph"

    def adapt_task(self, func: Callable[..., Any]) -> Callable[..., Any]:
        """LangGraph tasks map to graph nodes."""
        return func
